package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.Assignment;
import org.eclipse.jdt.internal.compiler.ast.CompoundAssignment;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.Reference;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class ThisReference extends Reference {
  public static org.eclipse.jdt.internal.compiler.ast.ThisReference implicitThis() {
    org.eclipse.jdt.internal.compiler.ast.ThisReference implicitThis = new org.eclipse.jdt.internal.compiler.ast.ThisReference(0, 0);
    implicitThis.bits |= 0x4;
    return implicitThis;
  }
  
  public ThisReference(int sourceStart, int sourceEnd) {
    this.sourceStart = sourceStart;
    this.sourceEnd = sourceEnd;
  }
  
  public FlowInfo analyseAssignment(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, Assignment assignment, boolean isCompound) {
    return flowInfo;
  }
  
  public boolean checkAccess(BlockScope scope, ReferenceBinding receiverType) {
    MethodScope methodScope = scope.methodScope();
    if (methodScope.isConstructorCall) {
      methodScope.problemReporter().fieldsOrThisBeforeConstructorInvocation((ASTNode)this);
      return false;
    } 
    if (methodScope.isStatic) {
      methodScope.problemReporter().errorThisSuperInStatic((ASTNode)this);
      return false;
    } 
    if (isUnqualifiedSuper()) {
      TypeDeclaration type = methodScope.referenceType();
      if (type != null && TypeDeclaration.kind(type.modifiers) == 2) {
        methodScope.problemReporter().errorNoSuperInInterface((ASTNode)this);
        return false;
      } 
    } 
    if (receiverType != null)
      scope.tagAsAccessingEnclosingInstanceStateOf(receiverType, false); 
    return true;
  }
  
  public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, int ttlForFieldCheck) {
    return true;
  }
  
  public void generateAssignment(BlockScope currentScope, CodeStream codeStream, Assignment assignment, boolean valueRequired) {}
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
    int pc = codeStream.position;
    if (valueRequired)
      codeStream.aload_0(); 
    if ((this.bits & 0x4) == 0)
      codeStream.recordPositionsFrom(pc, this.sourceStart); 
  }
  
  public void generateCompoundAssignment(BlockScope currentScope, CodeStream codeStream, Expression expression, int operator, int assignmentImplicitConversion, boolean valueRequired) {}
  
  public void generatePostIncrement(BlockScope currentScope, CodeStream codeStream, CompoundAssignment postIncrement, boolean valueRequired) {}
  
  public boolean isImplicitThis() {
    return ((this.bits & 0x4) != 0);
  }
  
  public boolean isThis() {
    return true;
  }
  
  public int nullStatus(FlowInfo flowInfo, FlowContext flowContext) {
    return 4;
  }
  
  public StringBuffer printExpression(int indent, StringBuffer output) {
    if (isImplicitThis())
      return output; 
    return output.append("this");
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    this.constant = Constant.NotAConstant;
    ReferenceBinding enclosingReceiverType = scope.enclosingReceiverType();
    if (!isImplicitThis() && !checkAccess(scope, enclosingReceiverType))
      return null; 
    this.resolvedType = (TypeBinding)enclosingReceiverType;
    MethodScope methodScope = scope.namedMethodScope();
    if (methodScope != null) {
      MethodBinding method = methodScope.referenceMethodBinding();
      if (method != null && method.receiver != null && TypeBinding.equalsEquals(method.receiver, this.resolvedType))
        this.resolvedType = method.receiver; 
    } 
    return this.resolvedType;
  }
  
  public void traverse(ASTVisitor visitor, BlockScope blockScope) {
    visitor.visit(this, blockScope);
    visitor.endVisit(this, blockScope);
  }
  
  public void traverse(ASTVisitor visitor, ClassScope blockScope) {
    visitor.visit(this, blockScope);
    visitor.endVisit(this, blockScope);
  }
}
